"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dakpan_1 = require("dakpan");
const react_1 = require("react");
const flattenTranslation_1 = require("./flattenTranslation");
exports.createLocalize = (language, languages) => {
    if (!languages.length) {
        throw new Error(`You may not provide 'createLocalize' with an empty 'languages' array`);
    }
    else if (languages.indexOf(language) === -1) {
        language = languages[0];
    }
    const { Provider, Consumer, withConsumer, context } = dakpan_1.createDakpan({
        language,
        languages,
        translations: {}
    })({
        setLanguage: (language) => () => ({
            language
        })
    });
    return {
        LocalizeProvider: Provider,
        withLocalize: withConsumer,
        withTranslations: (withTranslations) => (component) => (props) => react_1.createElement(Consumer, {
            children: ({ language, languages, translations }) => react_1.createElement(context.Provider, {
                value: {
                    language,
                    languages,
                    translations: Object.assign({}, translations, flattenTranslation_1.flattenTranslation(withTranslations))
                },
                children: react_1.createElement(component, props)
            })
        }),
        Translate: ({ id, data = {} }) => react_1.createElement(Consumer, {
            children: ({ language, languages, translations }) => {
                const index = languages.indexOf(language);
                const translation = translations[id];
                if (!translation) {
                    throw new Error(`Translation with id '${id}' not found`);
                }
                else if (index === -1 || translation.length <= index) {
                    throw new Error(`Language '${language}' for translation '${id}' not found`);
                }
                let result = translation[index];
                Object.keys(data).forEach((key) => result = result.replace(new RegExp('\\{' + key + '}', 'g'), data[key]));
                return result;
            }
        })
    };
};
//# sourceMappingURL=createLocalize.js.map